@@ -32,6 +32,9 @@
 
 #define TEMPLATE_REMATRIX_S16
 #include "rematrix_template.c"
+#define TEMPLATE_CLIP
+#include "rematrix_template.c"
+#undef TEMPLATE_CLIP
 #undef TEMPLATE_REMATRIX_S16
 
 #define TEMPLATE_REMATRIX_S32
@@ -65,8 +68,10 @@ int swr_set_matrix(struct SwrContext *s, const double *matrix, int stride)
     if (!s || s->in_convert) // s needs to be allocated but not initialized
         return AVERROR(EINVAL);
     memset(s->matrix, 0, sizeof(s->matrix));
-    nb_in  = av_get_channel_layout_nb_channels(s->user_in_ch_layout);
-    nb_out = av_get_channel_layout_nb_channels(s->user_out_ch_layout);
+    nb_in = (s->user_in_ch_count > 0) ? s->user_in_ch_count :
+        av_get_channel_layout_nb_channels(s->user_in_ch_layout);
+    nb_out = (s->user_out_ch_count > 0) ? s->user_out_ch_count :
+        av_get_channel_layout_nb_channels(s->user_out_ch_layout);
     for (out = 0; out < nb_out; out++) {
         for (in = 0; in < nb_in; in++)
             s->matrix[out][in] = matrix[in];
@@ -340,11 +345,16 @@ av_cold static int auto_matrix(SwrContext *s)
             }
     }
 
+    av_log(s, AV_LOG_DEBUG, "Matrix coefficients:\n");
     for(i=0; i<av_get_channel_layout_nb_channels(out_ch_layout); i++){
+        const char *c =
+            av_get_channel_name(av_channel_layout_extract_channel(out_ch_layout, i));
+        av_log(s, AV_LOG_DEBUG, "%s: ", c ? c : "?");
         for(j=0; j<av_get_channel_layout_nb_channels(in_ch_layout); j++){
-            av_log(NULL, AV_LOG_DEBUG, "%f ", s->matrix[i][j]);
+            c = av_get_channel_name(av_channel_layout_extract_channel(in_ch_layout, j));
+            av_log(s, AV_LOG_DEBUG, "%s:%f ", c ? c : "?", s->matrix[i][j]);
         }
-        av_log(NULL, AV_LOG_DEBUG, "\n");
+        av_log(s, AV_LOG_DEBUG, "\n");
     }
     return 0;
 }
@@ -362,18 +372,38 @@ av_cold int swri_rematrix_init(SwrContext *s){
             return r;
     }
     if (s->midbuf.fmt == AV_SAMPLE_FMT_S16P){
+        int maxsum = 0;
         s->native_matrix = av_calloc(nb_in * nb_out, sizeof(int));
         s->native_one    = av_mallocz(sizeof(int));
-        for (i = 0; i < nb_out; i++)
-            for (j = 0; j < nb_in; j++)
-                ((int*)s->native_matrix)[i * nb_in + j] = lrintf(s->matrix[i][j] * 32768);
+        if (!s->native_matrix || !s->native_one)
+            return AVERROR(ENOMEM);
+        for (i = 0; i < nb_out; i++) {
+            double rem = 0;
+            int sum = 0;
+
+            for (j = 0; j < nb_in; j++) {
+                double target = s->matrix[i][j] * 32768 + rem;
+                ((int*)s->native_matrix)[i * nb_in + j] = lrintf(target);
+                rem += target - ((int*)s->native_matrix)[i * nb_in + j];
+                sum += FFABS(((int*)s->native_matrix)[i * nb_in + j]);
+            }
+            maxsum = FFMAX(maxsum, sum);
+        }
         *((int*)s->native_one) = 32768;
-        s->mix_1_1_f = (mix_1_1_func_type*)copy_s16;
-        s->mix_2_1_f = (mix_2_1_func_type*)sum2_s16;
-        s->mix_any_f = (mix_any_func_type*)get_mix_any_func_s16(s);
+        if (maxsum <= 32768) {
+            s->mix_1_1_f = (mix_1_1_func_type*)copy_s16;
+            s->mix_2_1_f = (mix_2_1_func_type*)sum2_s16;
+            s->mix_any_f = (mix_any_func_type*)get_mix_any_func_s16(s);
+        } else {
+            s->mix_1_1_f = (mix_1_1_func_type*)copy_clip_s16;
+            s->mix_2_1_f = (mix_2_1_func_type*)sum2_clip_s16;
+            s->mix_any_f = (mix_any_func_type*)get_mix_any_func_clip_s16(s);
+        }
     }else if(s->midbuf.fmt == AV_SAMPLE_FMT_FLTP){
         s->native_matrix = av_calloc(nb_in * nb_out, sizeof(float));
         s->native_one    = av_mallocz(sizeof(float));
+        if (!s->native_matrix || !s->native_one)
+            return AVERROR(ENOMEM);
         for (i = 0; i < nb_out; i++)
             for (j = 0; j < nb_in; j++)
                 ((float*)s->native_matrix)[i * nb_in + j] = s->matrix[i][j];
@@ -384,6 +414,8 @@ av_cold int swri_rematrix_init(SwrContext *s){
     }else if(s->midbuf.fmt == AV_SAMPLE_FMT_DBLP){
         s->native_matrix = av_calloc(nb_in * nb_out, sizeof(double));
         s->native_one    = av_mallocz(sizeof(double));
+        if (!s->native_matrix || !s->native_one)
+            return AVERROR(ENOMEM);
         for (i = 0; i < nb_out; i++)
             for (j = 0; j < nb_in; j++)
                 ((double*)s->native_matrix)[i * nb_in + j] = s->matrix[i][j];
@@ -395,6 +427,8 @@ av_cold int swri_rematrix_init(SwrContext *s){
         // Only for dithering currently
 //         s->native_matrix = av_calloc(nb_in * nb_out, sizeof(double));
         s->native_one    = av_mallocz(sizeof(int));
+        if (!s->native_one)
+            return AVERROR(ENOMEM);
 //         for (i = 0; i < nb_out; i++)
 //             for (j = 0; j < nb_in; j++)
 //                 ((double*)s->native_matrix)[i * nb_in + j] = s->matrix[i][j];
